//
//  Game_Object.cpp
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include <zenilib.h>
#include "Game_Object.h"
#include "Track.h"

using namespace std;
using namespace Zeni;

Game_Object::Game_Object() {}

Game_Object::Game_Object(const Point2f &position_, const Vector2f &size_, const float &theta_, const float & speed_):
  m_position(position_),
  m_size(size_),
  m_theta(theta_),
  m_speed(speed_)
  {
    
  }
  
  
void Game_Object::turn_left(const float &theta_){
    m_theta += theta_;
  }
  
  void Game_Object::move_forward(const float &move_){
    m_position.x += move_ * cos(m_theta);
    m_position.y += move_ * -sin(m_theta);
  }
  
  
  //BULLET STUFF
  const Point2f & Game_Object::get_position() const {return m_position;}
  const Vector2f & Game_Object::get_size() const {return m_size;}
  const float & Game_Object::get_theta() const {return m_theta;}
  
  const float Game_Object::get_radius() const {
    return 0.5f * m_size.magnitude();
  }
  
  //COLLISION DETECTION
  bool Game_Object::collide(const Game_Object & rhs) const{
    
    //calculate distance between
    const Vector2f dist_vec = m_position - rhs.m_position +
    0.5f * (m_size - rhs.m_size);
    
    //make sure it is not negative
    const float dist = sqrt(dist_vec * dist_vec);
    
    //if the distance is less than the summation of their radii,
    //we have a collision!
    return dist < get_radius() + rhs.get_radius();
  }
  
  void Game_Object::render(const String &texture, const Color &filter) const{
    //Use a helper defined in Zeni/EZ2D.h
    render_image(
                 texture, //which texture to use
                 m_position, //upper-left corner
                 m_position + m_size, //lower-right corner
                 m_theta, //rotation in radians
                 1.0f, //scaling factor
                 m_position + 0.5f * m_size, //point to rotate and scale about
                 false, //whether or not to horizontally flip the texture
                 filter); //What Color to "paint" the texture
  }
  
  




