//
//  Level.cpp
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
#include <zenilib.h>
#include "Level.h"
#include <fstream>
#include <list>

Level::Level(const int & levelNum){
  levelNumber = levelNum;
  int inputNumTracks = 0;
  int inputNumTrackPieces = 0;
  int inputNumStartingCars = 0;
  
  for(int i=0; i< 30; i++){
    lastX[i] = 0;
  }
  
  //OPEN INPUT FILE

  ifstream infile;
  ifstream rollingIn;

  
  switch(levelNum){
    case 1:
      infile.open("level1.txt");
      rollingIn.open("rolling_stock1.txt");

      break;
    case 2:
      infile.open("level2.txt");
      rollingIn.open("rolling_stock2.txt");

      break;
    case 3:
      infile.open("level3.txt");
      rollingIn.open("rolling_stock3.txt");
      break;
    default:
      break;
  }
  
  if(rollingIn.fail()){
    cout<<"FAIL"<<endl;
    assert(0);
  }
  if(infile.fail()){
    cout<<"FAIL"<<endl;
    assert(0);
  }
  
  //READ IN BASIC DATA
  infile>>inputNumTracks;
  infile>>inputNumTrackPieces;
  infile>>inputNumStartingCars;
  
  //ERROR CHECK
  if(inputNumTracks == 0 || inputNumTrackPieces == 0){
    assert(0);
  }
  
  numTracks = inputNumTracks;
  numTrackPieces = inputNumTrackPieces;
  levelEnd = numTrackPieces*TRACK_WIDTH;
  
  //ADD TRACKS TO MAP
  for(int j=0; j<numTracks; j++){
    Track newTrack(j+1, 50.0f+j*TRACK_SPACING, numTrackPieces);
    tracks[j] = newTrack;
  }
  
  
  //READ IN TRACK PIECE DATA FROM FILE
  char string[10];
  infile.getline(string, 10, '\n');
  for(int k=0; k< numTrackPieces; k++){ // For every track piece (row in input)
    infile.getline(string, 10, '\n');
    
    for (int m = 0; m<numTracks; m++){ //For every Track
      char A = string[numTracks-m-1];
      if (A == '|')
        tracks[m].addTrackPiece(k, STRAIGHT);
      else if(A == '>')
        tracks[m].addTrackPiece(k, SWITCH_UP);
      else if(A == '<')
        tracks[m].addTrackPiece(k, SWITCH_DOWN);
      else if(A == 'i')
        tracks[m].addTrackPiece(k, INVISIBLE);
      else if(A == 'S')
        tracks[m].addTrackPiece(k, STATION);
      else if(A == 'X')
        tracks[m].addTrackPiece(k, ENDOFLINE);
      else
        cout<<"here I am"<<endl;
    }
  }
  
  //INIT PLAYER TRAIN
  playerTrain = new Train(this, 3, Point2f(100.0f, 100.0f), Vector2f(50.0f, 25.0f), 0);
  playerTrain->addToTrack(tracks[2], 700.0f);
  
  for(int i=0; i<10; i++){
    if(i<inputNumStartingCars && levelNumber != 3)
      playerTrain->addRealCar(CATTLE);
  }
  
  
  
  
  vector<Car> carList;
  carList.push_back(Car(GATOR)); 
  carList.push_back(Car(CATTLE)); 
  carList.push_back(Car(SHEEP)); 
  carList.push_back(Car(CATTLE)); 
  carList.push_back(Car(CATTLE)); 
  carList.push_back(Car(CHICKEN)); 
  carList.push_back(Car(ROOSTER)); 
  
  
  
  //READ IN ROLLING STOCK INFO
  //OPEN INPUT FILE

  
  int numRollingStock = 0;
  rollingIn>>numRollingStock;
  
  //READ IN TROLLING STOCK DATA FROM FILE
  char line[50] = {' '};
  rollingIn.getline(line, 50, '\n');
  for(int k=0; k< numRollingStock; k++){ // For every rolling stock piece (row)
    char line[50] = {' '};
    rollingIn.getline(line, 50, '\n');
    float* timeTrigger = new float();
    char locoOrCar[20];
    char type[20];
    char carType[20];
    int* trackAssignment = new int();
    int sscanfResult = sscanf(line, "%f%s%s%s%i",  timeTrigger, locoOrCar, type, carType, trackAssignment);
    if(sscanfResult == -1){
      cerr<<"sscanf didn't work"<<endl;
      assert(0);
    }
    
    if(!strcmp(locoOrCar, "LOCO")){
      if(!strcmp(type, "SLOW")){
        //Track/DC/ speed / cars / trigger / offset
        if(! strcmp(carType, "REVERSE"))
          addNewAITrain(*trackAssignment, 100.0f, -20.0f, carList, *timeTrigger, 1200.0f);
        else
          addNewAITrain(*trackAssignment, 100.0f, 90.0f, carList, *timeTrigger, -500.0f);
      }
      else if( ! strcmp(type, "FAST")){
        if( ! strcmp(carType, "REVERSE"))
          addNewAITrain(*trackAssignment, 100.0f, -200.0f, carList, *timeTrigger, 2000.0f);
        else
          addNewAITrain(*trackAssignment, 100.0f, 300.0f, carList, *timeTrigger, -1000.0f);
      }
      else if( ! strcmp(type, "DROPBACK")){
        addNewAITrain(*trackAssignment, 100.0f, 30.0f, carList, *timeTrigger, 1500.0f);
      }
    }
    
    if(!strcmp(locoOrCar, "CAR")){
      Car_Type carEnum = stringToEnum(carType);
      if(!strcmp(type, "STANDARD")){
        //Track/DC/ speed / cars / trigger / offset
        addNewParkedCar(*trackAssignment, 100.0f, 20.0f, carEnum, *timeTrigger, 1000.0f);
      }
    }
    
    delete timeTrigger;
    delete trackAssignment;
    
  }
  
  
  
  
}

Car_Type stringToEnum(char* string){
  if( ! strcmp(string,  "CHICKEN")){
    return CHICKEN;
  }
  else if (! strcmp(string, "CATTLE")){
    return CATTLE;
  }
  else if (! strcmp(string, "SHEEP")){
    return SHEEP;
  }
  else if (! strcmp(string, "ROOSTER")){
    return ROOSTER;
  }
  else{
    return CHICKEN;
  }
}

void Level::addNewParkedCar(const int track_Number_, const float xCoord_, const float v_0, Car_Type type_, const float timeTrigger_, float offset){
  Parked_Car* trainCar = new Parked_Car(this, track_Number_, xCoord_, v_0, type_, timeTrigger_);
  trainCar->offset = offset;
  waitingParkedCars.push_back(trainCar);
}

void Level::addNewAITrain(const int trackNumber_, const float xCoord_, const float v_0, vector<Car> & cars, const float timeTrigger_, float offset){
  AI_Train* compTrain = new AI_Train(this, trackNumber_, xCoord_, v_0, cars, timeTrigger_);
  compTrain->offset = offset;
  waitingAITrains.push_back(compTrain);
}

void Level::triggerForX(const float X){
  float gatorFactor = 1;
  
  if(levelNumber == 3)
    gatorFactor = 3;
  
  if(abs(lastX[0] - X) > 1100*gatorFactor){
    addNewParkedCar(1, X, 20.0f, CATTLE, X, 1000);
    lastX[0] = X;
  }
  
  
  if(abs(lastX[1] - X) > 1697*gatorFactor){
    addNewParkedCar(2, X, 20.0f, CATTLE, X, 1015);
    lastX[1] = X;
  }
  
  
  if(abs(lastX[3] - X) > 750*gatorFactor){
    addNewParkedCar(3, X, 20.0f, SHEEP, X, 998);
    lastX[3] = X;
  }
  
  if(abs(lastX[4] - X) > 1800*gatorFactor){
    addNewParkedCar(2, X, 20.0f, SHEEP, X, 1006);
    lastX[4] = X;
  }
  
  
  if(abs(lastX[5] - X) > 1505*gatorFactor){
    addNewParkedCar(4, X, 20.0f, SHEEP, X, 1100);
    lastX[5] = X;
  }
  
  if(abs(lastX[6] - X) > 1130*gatorFactor){
    addNewParkedCar(4, X, 20.0f, CHICKEN, X, 1000);
    lastX[6] = X;
  }
  
  if(abs(lastX[7] - X) > 600*gatorFactor){
    addNewParkedCar(5, X, 20.0f, CHICKEN, X, 1000);
    lastX[7] = X;
  }
  
  if(abs(lastX[8] - X) > 1906*gatorFactor){
    addNewParkedCar(6, X, 20.0f, CHICKEN, X, 1000);
    lastX[8] = X;
  }
  
  if(abs(lastX[9] - X) > 2457*gatorFactor){
    addNewParkedCar(5, X, 40.0f, ROOSTER, X, 1030);
    lastX[9] = X;
  }
  
  if(abs(lastX[10] - X) > 640*gatorFactor){
    addNewParkedCar(7, X, 20.0f, CHICKEN, X, 1000);
    lastX[10] = X;
  }
  
  if (levelNumber == 3){
    if(abs(lastX[11] - X) > 640){
      addNewParkedCar(1, X, 30.0f, GATOR, X, 1000);
      lastX[11] = X;
    }
    
    if(abs(lastX[12] - X) > 956){
      addNewParkedCar(2, X, 20.0f, GATOR, X, 1000);
      lastX[12] = X;
    }
    
    if(abs(lastX[13] - X) > 1553){
      addNewParkedCar(3, X, 35.0f, GATOR, X, 1000);
      lastX[13] = X;
    }
    
    if(abs(lastX[14] - X) > 3535){
      addNewParkedCar(4, X, 20.0f, GATOR, X, 1000);
      lastX[14] = X;
    }
    
    if(abs(lastX[15] - X) > 1645){
      addNewParkedCar(5, X, 40.0f, GATOR, X, 1000);
      lastX[15] = X;
    }
    
    if(abs(lastX[16] - X) > 936){
      addNewParkedCar(6, X, 20.0f, GATOR, X, 1000);
      lastX[16] = X;
    }
    
    if(abs(lastX[17] - X) > 600){
      addNewParkedCar(7, X, 10.0f, GATOR, X, 1000);
      lastX[17] = X;
    }
    
  }
  
}

void Level::triggerForTime(const float time, bool* alerts){
  triggerStuff(waitingAITrains, AITrains, time, alerts);
  triggerStuff(waitingParkedCars, parkedCars, time, alerts);
  
}

void Level::filterAITrains(){
  float leftBoundary = playerTrain->m_position.x - 1000.0f;
  float rightBoundary = playerTrain->m_position.x + 2500.0f;
  
  for(list<AI_Train *>::iterator it = AITrains.begin(); it != AITrains.end(); ++it){   
    if((*it)->m_position.x < leftBoundary && (*it)->m_speed < 0 ||
       (*it)->m_position.x > rightBoundary && (*it)->m_speed > 40.0f ||
       abs((*it)->m_speed) <= 10){
      delete (*it);
      it = AITrains.erase(it);
    }
  }
}

void Level::filterParkedCars(){
  float leftBoundary = playerTrain->m_position.x - 1200.0f;
  
  for(list<Parked_Car *>::iterator it = parkedCars.begin(); it != parkedCars.end(); ++it){   
    if((*it)->m_position.x < leftBoundary){
      delete (*it);
      it = parkedCars.erase(it);
    }
  }
}


template<typename T>
void Level::triggerStuff(list<T*> & waitingStuff, list<T* > & activeStuff, float time, bool*alerts){
  if ( ! waitingStuff.empty()){
    T* candidate = NULL;
    candidate = waitingStuff.front();
    while(candidate->timeTrigger <= time){
      waitingStuff.pop_front();
      
      candidate->m_position.x = playerTrain->m_position.x + candidate->offset;
      
      //only issue alarm for AI Trains, trigger alarm based on offset
      if(candidate->causeForAlarm){
        if(candidate->offset > 0){
          alerts[candidate->track->number-1] = true;
        }
        else{
          alerts[candidate->track->number + 4] = true;
        }
      }
      
      //If it's going to spawn beyond the end of the level, delete it
      if(candidate->m_position.x > levelEnd){
        delete candidate;
      }
      
      //If it's going to spawn in negative land, put it at 0
      else if(candidate->m_position.x < 0.0f){
        candidate->m_position.x = 0.0f;
        activeStuff.push_back(candidate);
      }
      
      //if it's a normal average Joe, then leave it be
      else {
        //Add it to the list of current trains
        activeStuff.push_back(candidate);
      }
      
      candidate = waitingStuff.front();
      
      if (waitingStuff.empty())
        break;
    }
  }
  
}



//HANDLE UNSPAWNED PARKED CARS

void Level::render() const{ 
  for(int i = 0; i< numTracks; i++){
    tracks[i].render();
  }
}

Level::~Level(){
  delete playerTrain;
  for(list<Parked_Car *>::iterator it = parkedCars.begin(); it != parkedCars.end(); ++it){   
    delete (*it);
  }
  for(list<AI_Train *>::iterator it = AITrains.begin(); it != AITrains.end(); ++it){   
    delete (*it);
  }
}


