//
//  Track.cpp
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//


#include <zenilib.h>
#include "Game_Object.h"
#include "Track.h"

using namespace std;
using namespace Zeni;

Track::Track() { }

Track::Track(const Track & track){
  copyAll(track);
}

Track& Track::operator=(const Track & track){
  if(this!=&track){
    copyAll(track);
  }
  return *this;
}

Track:: Track(int assignedNumber, float horizontalCoord, int numTrackPieces_) {
  number = assignedNumber;
  
  yCoord = horizontalCoord;
  
  numTrackPieces = numTrackPieces_;
  trackPieces = new TrackPiece[numTrackPieces];
}

void Track::addTrackPiece(int index, TrackTypeEnum type_){
  Point2f position_(index*TRACK_WIDTH, yCoord);
  TrackPiece track(position_, type_);
  if(type_ == STATION){
    stationLocations.push_back(position_);
  }
  trackPieces[index] = track;
}

void Track::setViewWindow(Point2f upperLeftPoint, Point2f lowerRightPoint){
  upperLeftCorner = upperLeftPoint;
  lowerRightCorner = lowerRightPoint;
}


//Converts a global XCoord into a local piece-view xcoord and asks the given track
//what to do
void Track::moveAlong(TrainInfoPack & trainData){
  int trackPieceNum = calculateTrackPieceNum(trainData.position.x);
  float localX = calculateLocalX(trainData.position.x);
  trackPieces[trackPieceNum].moveAlong(trainData, localX);
}

int Track::calculateTrackPieceNum(float X){
  return X/TRACK_WIDTH;
}

float Track::calculateLocalX(float X){
  return (int)X % (int)TRACK_WIDTH;
}

//return number of car to be removed
int Track::checkForStationScore(float xCoord){
  int trackPieceNum = calculateTrackPieceNum(xCoord);
  if(trackPieces[trackPieceNum].type == STATION){
    return true;
  }
  else return false;
}



void Track::render() const{ 
  for(int i = 0; i< numTrackPieces; i++){
    trackPieces[i].render();
  }

}

void Track::copyAll(const Track & track){  
  this->numTrackPieces = track.numTrackPieces;
  this->upperLeftCorner = track.upperLeftCorner;
  this->lowerRightCorner = track.lowerRightCorner;
  this->yCoord = track.yCoord;
  this->number = track.number;
  trackPieces = new TrackPiece[numTrackPieces];
  for(int i=0; i< numTrackPieces; i++)
    trackPieces[i] = track.trackPieces[i];
    
}
  
Track::~Track(){
  delete[] trackPieces;
}








TrackPiece::TrackPiece() {}

TrackPiece::TrackPiece(const Point2f &position_, TrackTypeEnum type_) {
  m_theta = 0.0f;
  type = type_;
  switchedAtX = 0;
  setSwitch = false;
  
  switch (type_){
    case STRAIGHT:
      m_position = position_;
      m_size = Vector2f(TRACK_WIDTH, TRACK_HEIGHT);
      break;
      
    case SWITCH_UP:
      m_position = Point2f(position_.x, position_.y - TRACK_SPACING);
      m_size = Vector2f(TRACK_WIDTH, TRACK_HEIGHT + TRACK_SPACING);
      break;
      
    case SWITCH_DOWN:
      m_position = position_;
      m_size = Vector2f(TRACK_WIDTH, TRACK_HEIGHT + TRACK_SPACING);
      break;
      
    case STATION:
      m_position = position_;
      m_size = Vector2f(TRACK_WIDTH, TRACK_HEIGHT);
      break;
      
    default:
      m_position = position_;
      m_size = Vector2f(TRACK_WIDTH, TRACK_HEIGHT);
      break;
  }
}

void TrackPiece::render() const {
  switch (type){
    case STRAIGHT:
      Game_Object::render("Track_Straight");
      break;
      
    case SWITCH_DOWN:
      Game_Object::render("Track_Switch_Down");
      break;
      
    case SWITCH_UP:
      Game_Object::render("Track_Switch_Up");
      break;
      
    case STATION:
      Game_Object::render("Track_Straight");
      break;
      
    case ENDOFLINE:
      Game_Object::render("Track_EndOfLine");
      break;
      
    case INVISIBLE:
      break;
      
    default:
      Game_Object::render("Track_Straight");
      break;
  }
}

void TrackPiece::moveAlong(TrainInfoPack & trainData, float localX){
  int switchThreshold = 0;

  
  
  
  
  //CHECK FOR TERMINAL VELOCITY, ADJUST A ACCORDINGLY
  if (trainData.v >= trainData.v_term && trainData.isPlayerTrain && trainData.a >0) {
    trainData.a = -1.0f*(trainData.v - trainData.v_term); 
  }
  
  //CALCULATE CHANGE IN VELOCITY
  float Delta_V = trainData.a * trainData.t;
  
  //CALCULATE CHANGE IN DISPLACEMENT
  float Delta_X = 0.5*trainData.a*pow(trainData.t, 2)  +  trainData.v * trainData.t;
  if(Delta_X < 0 && trainData.isPlayerTrain){
    Delta_X = 0.0f;
    trainData.a = 0;
    trainData.v = 0;
  }
  
  
  
  
  switch(type){
    case ENDOFLINE:
      trainData.v = ((TRACK_WIDTH - localX)/TRACK_WIDTH)*trainData.v;
      trainData.position.x += trainData.v*trainData.t;
      return;
      break;
      
    case SWITCH_DOWN:
    case SWITCH_UP:
      if ((trainData.command == SWITCH_DOWN && type==SWITCH_DOWN || 
           trainData.command == SWITCH_UP && type == SWITCH_UP) && 
          canSwitch(localX) && trainData.isLocomotive && setSwitch == false){
        //switchedAtX = localX;
        setSwitch = true;
      }
      
      //If switched, follow the switch with trig!
      if(setSwitch && localX > switchThreshold && trainData.isPlayerTrain){
        float X_Projection = Delta_X * cos(30*Global::pi/180);
        float Y_Projection = Delta_X * sin(30*Global::pi/180);
        
        if(type == SWITCH_UP)
          Y_Projection = - Y_Projection;
        trainData.position.x = trainData.position.x + X_Projection;
        trainData.position.y = trainData.position.y + Y_Projection;
        float theta = getTheta(localX+X_Projection);
        if (type==SWITCH_UP)
          theta = -theta;
        trainData.theta = theta;
        
        //Don't go past the next track
        
        if(trainData.position.y >= m_position.y + TRACK_SPACING && type == SWITCH_DOWN){
          trainData.position.y  = m_position.y + TRACK_SPACING;
        }
        else if(trainData.position.y <= m_position.y && type == SWITCH_UP){
          trainData.position.y  = m_position.y;
        }
        
        //If You reach almost to the end of a track piece, mark completed Switch
        if(localX > 0.95*TRACK_WIDTH){
          trainData.completedSwitch = true;
        }
      }
      else{
        trainData.position.x = trainData.position.x + Delta_X;
      }
      
      
      break;
      
    default:  // STRAIGHT TRACKS AND STATIONS
      trainData.completedSwitch = true;
      trainData.position.x = trainData.position.x + Delta_X;
      break;
  }

  
  
  //UPDATE VELOCITY
    trainData.v = trainData.v + Delta_V;
  
  //CHECK IF VELOCITY IS 0 FOR PLAYER TRAIN
    if (trainData.v <= 0 && trainData.isPlayerTrain){
      trainData.a = 0.0f;
      trainData.v = 0.0f;
    }
  
}



bool TrackPiece::canSwitch(float localX){
  if (localX > 0.0*TRACK_WIDTH && localX < 0.05*TRACK_WIDTH){
      //(type == SWITCH_UP || type == SWITCH_DOWN)) {
    return true;
  }
  else return false;
}


float TrackPiece::getTheta(float localX){
  /*
  if (localX < 0.3 * TRACK_WIDTH){
    return 0;//(45*Global::pi/180)/(WIDTH/4) * localX + 90*Global::pi/180;
  }
  else if (localX > 0.7 * TRACK_WIDTH){
    return 0;
  }
  else{
    return -30*Global::pi/180;
  }
  
  */
  
   
    float intercept = 0;
    float c = -120*(intercept-pow(intercept,2)) / (4*intercept - 4*pow(intercept,2) - 1);
    float a = 4*c + 120;
    float b = -a;
    float result = Global::pi/180*(a*pow(localX/TRACK_WIDTH+0.1, 2) + b*(localX+0.1)/TRACK_WIDTH + c);
    //30*Global::pi/180; //-(45*Global::pi/180)/(WIDTH/4) * localX + Global::pi + 90*Global::pi/180;
    //}
    if (result > 0)
      result = 0;
  return result;
   
}

TrackPiece::~TrackPiece() {}


/*
SwitchPiece::SwitchPiece(const Point2f &position_, const Vector2f &size_, 
                         const float &theta_, bool allowUpSwitch, bool allowDownSwitch)
{
  switchDown = allowDownSwitch;
  switchUp = allowUpSwitch;
}

*/




