//
//  Train.cpp
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
#include <zenilib.h>
#include "Train.h"
#include "Track.h"
#include "Level.h"

string DebugOutput1;
string DebugOutput2;
string DebugOutput3;
string DebugOutput4;

Car::Car(){ }

Car::Car(Level* level_, const Point2f &position_,  const float &theta_, const float timeTrigger_)
: Game_Object(position_, CAR_SIZE, theta_),
reverse(false),
timeTrigger(timeTrigger_)
{
  level = level_;
  addToNearestTrack(level_);
  causeForAlarm = false;
}

Car::Car(Car_Type type_)
: Game_Object(Point2f(0.0f,0.0f), CAR_SIZE, 0.0f),
reverse(false)
{
  type = type_;
}

int Car::addToTrack(Track & track_, float verticalCoord){
  track = &track_;
  m_position.x = verticalCoord;
  m_position.y = track->yCoord;  // + track.TRACK_HEIGHT/2;
  return 1;
}

void Car::addToNearestTrack(Level* level_){
  float max = 1000.0f;
  int trackDesired = 0;
  
  for (int i=0; i< level->numTracks; i++){
    float distance = abs(level_->tracks[i].yCoord - m_position.y);
    if (distance < max){
      trackDesired = i;
      max = distance;
    }
  }
  addToTrack(level_->tracks[trackDesired], m_position.x);
}


void Car::addToNewTrack(Track * track_){
  track = track_;
  m_position.y = track->yCoord;
}


void Car::moveToPosition(float X, float Y, float theta){
  m_position.x = X;
  m_position.y = Y;
  m_theta = theta;
  
}

bool Car::isOnCurve(){
  int trackNumber = m_position.x / TRACK_WIDTH;
  if (track->yCoord != m_position.y &&
      track->trackPieces[trackNumber].type != STRAIGHT){
    return true;
  }
  else return false;
}

const float Car::get_radius() const{
  return 0.5f * m_size.magnitude();
}

float Car::get_radiusForAngle(float alpha) const{
  
  const float phi = asin((CAR_HEIGHT/2) /
                         sqrt(pow(CAR_LENGTH/2, 2) + pow(CAR_HEIGHT/2,2)));
  if (alpha < phi){
    return (CAR_LENGTH/2)/cos(alpha);
  }
  else
    return (CAR_HEIGHT/2)/sin(alpha);
  
}

bool Car::collide(const Car & rhs) const{
  //calculate distance between
  //Point2f leadingEdge = m_position + m_size;
  
  //const float x_distance = m_position.x - rhs.m_position.x;
  
  
  
  const Vector2f dist_vec = m_position - rhs.m_position +
  0.5f * (m_size - rhs.m_size);
  
  float alpha = abs(atan(dist_vec.y/dist_vec.x));
  
  //make sure it is not negative
  const float dist = sqrt(dist_vec * dist_vec);
  
  //if the distance is less than the summation of their radii,
  //we have a collision!
  return dist < get_radiusForAngle(alpha) + rhs.get_radiusForAngle(alpha);
}

bool Car::heuristic_collide(const Car & rhs) const{
  if(track->number - rhs.track->number < 2){
    cout<<type<<" Car is at "<<m_position.x<<endl;
    cout<<"You are at "<<rhs.m_position.x<<endl;
    if(abs(m_position.x - rhs.m_position.x) < 60.0f){
      cout<<"TRUE TRUEdf TRUE"<<endl;
      return true;
    }
    cout<<"NO"<<endl;
  }
  return false;
}

void Car::render() const{
  switch(type){
    case INVIS:
      break;
    case CATTLE:
      Game_Object::render("Car_Cattle");
      break;      
    case SHEEP:
      Game_Object::render("Car_Sheep");
      break;            
    case ROOSTER:
      Game_Object::render("Car_Rooster");
      break;      
    case CHICKEN:
      Game_Object::render("Car_Chicken");
      break;     
    case GATOR:
      Game_Object::render("Gator");
      break;      
    default:
      Game_Object::render("Car_Cattle");
      break;
  }
}


void Car::moveCarAlong(TrainInfoPack & trainData){
  trainData.position = m_position;
  trainData.v = m_speed;
  trainData.theta = m_theta;
  
  track->moveAlong(trainData);
  m_position = trainData.position;
  m_speed = trainData.v;
  m_theta = trainData.theta;
  
  /*
   if(trainData.completedSwitch){
   cout<<"Second"<<endl;
   addToNearestTrack(level);
   }
   */
  
}








/*
 Parked_Car::Parked_Car(const Point2f &position_, const Vector2f &size_, const float &theta_)
 :Car(position_, size_, theta_)
 {
 }
 */

Parked_Car::  Parked_Car(Level* level_, const int trackNumber_, const float xCoord_, const float v_0, Car_Type type_, const float timeTrigger_)
:Car(level_, Point2f(0.0f, 0.0f), 0.0f, timeTrigger_)
{
  type = type_;
  addToTrack(level_->tracks[trackNumber_-1], xCoord_);
  type = type_;
  m_speed = v_0;
}

/*
 void Parked_Car::render() const{
 Game_Object::render("Car_Cattle");
 }
 */








Train::Train() { } 

Train::Train(Level* level_, const int trackNumber_, const Point2f &position_, const Vector2f &size_, const float &theta_)
: Car(level_, position_, theta_, 0.0f),
crashed(false),
numCars(0),
numVisibleCars(0)
{
  level = level_;
  addToNewTrack(&(level_->tracks[trackNumber_-1]));
  
  for(int i=0; i<12; i++){
    spawnCar(INVIS);
  }
}


void Train::collide( list<Parked_Car *> & parkedCars){ 
  
  //TEST COLLISIONS WITH EACH PARKED CAR
  for(list<Parked_Car *>::iterator it = parkedCars.begin(); it != parkedCars.end(); ++it){
    
    //FIRST TEST LOCOMOTIVE
    if(heuristic_collide(**it)){
      if((*it)->collide(*this)) {
        addCar((**it).type);
        delete (*it);
        it = parkedCars.erase(it);
        break;
      }
    }
    
    /*
     //NEXT TRY EACH CAR
     for(int i=0; i< numVisibleCars; i++){
     if(cars[i].heuristic_collide(**it)){
     if((*it)->collide(cars[i])) {
     addCar((**it).type);
     delete (*it);
     it = parkedCars.erase(it);
     break;
     }
     }
     }
     */
  }
}

bool Train::collide(list<AI_Train *> & AITrains){
  for(list<AI_Train *>::iterator it = AITrains.begin(); it != AITrains.end(); ++it){
    //FIRST TEST LOCOMOTIVE
    if((*it)->heuristic_collide(*this)){
      if((*it)->collide(*this)){
        delete (*it);
        it = AITrains.erase(it);
        return true;
        break;
      }
    }
    
    //NEXT TRY EACH CAR
    for(int i=0; i< numVisibleCars; i++){
      if(cars[i].heuristic_collide(**it)){
        if((*it)->collide(cars[i])) {
          delete (*it);
          it = AITrains.erase(it);
          return true;
          break;
        }
      }
    }
  }
  return false;
}

void Train::moveAlongTrack(TrainInfoPack & trainData){
  trainData.isLocomotive = true;
  moveCarAlong(trainData);
  if(trainData.completedSwitch){
    cout<<"Second"<<endl;
    addToNearestTrack(level);
  }
  moveCars(trainData);
  
  //Correct first car for spacing issues
  //TODO:LOOK AT THIS
  
  if (numCars>0)
    cars[0].m_speed = m_speed;
  if(!isOnCurve() && !cars[0].isOnCurve()){
    if (!reverse){
      cars[0].addToTrack(*track, cars[0].m_position.x - CAR_LENGTH - CAR_SPACING);
      cars[0].m_position.x = m_position.x - CAR_LENGTH - CAR_SPACING;
      cars[0].m_position.y = m_position.y;
    }
    else{
      cars[0].addToTrack(*track, cars[0].m_position.x + CAR_LENGTH + CAR_SPACING);
      cars[0].m_position.x = m_position.x + CAR_LENGTH + CAR_SPACING;
      cars[0].m_position.y = m_position.y;
    }
    /*
     if (!reverse)
     cars[0].addToTrack(*track, cars[0].m_position.x - CAR_LENGTH - CAR_SPACING);
     else
     cars[0].addToTrack(*track, cars[0].m_position.x + CAR_LENGTH + CAR_SPACING);
     */
  }
}


void Train::spawnCar(Car_Type type_){
  Car car(level, m_position, m_theta, 0.0f);
  car.type = type_;
  car.track = track;
  if(!reverse)
    car.moveToPosition(m_position.x - (numCars+1)*(CAR_LENGTH + CAR_SPACING), m_position.y, m_theta);
  else
    car.moveToPosition(m_position.x + (numCars+1)*(CAR_LENGTH + CAR_SPACING), m_position.y, m_theta+Global::pi);
  car.level = level;
  cars.push_back(car);
  numCars++;
  if(type_ != INVIS)
    numVisibleCars++;
}

void Train::addRealCar(Car_Type type_){
	if(numVisibleCars > 0 && numVisibleCars != numCars){
		for(int i=0; i< numCars; i++){
			if(cars[i].type == INVIS){
				cars[i].type = type_;
				numVisibleCars++;
				break;
			}
		}
	}
	else if(numVisibleCars ==0){
		cars[numVisibleCars].type = type_;
		numVisibleCars++;
	}
}

void Train::addCar(Car_Type type_){
  if(numVisibleCars < MAX_CARS){
    addRealCar(type_);
  }
}

Car_Type Train::removeCar(int index){
  Car_Type result = NONE;
  if(numVisibleCars > 0){
    result = cars[index].type;
    cars[index] .type = INVIS;
    numVisibleCars--;
  }
  return result;
}

void Train::moveCars(TrainInfoPack & trainData){
  //TRAIN DATA RESET
  
  /*
   string str = itoa(track->number);
   string str2 = itoa(cars[1].track->number);
   string str3 = itoa(cars[2].track->number);
   string str4 = itoa(cars[3].track->number);
   
   DebugOutput1.assign(str);
   DebugOutput2.assign(str2);
   DebugOutput3.assign(str3);
   DebugOutput4.assign(str4);
   */
  
  
  for(int i=0; i<numCars; i++){
    trainData.isLocomotive = false;
    trainData.position.x = 0;
    trainData.v = 0;
    trainData.completedSwitch = false;
    
    cars[i].moveCarAlong(trainData);
    
    //AFTER SWITCH, CORRECT CAR'S POSITION
    if (trainData.completedSwitch){
      cars[i].addToNearestTrack(level);
    }
    
    //Correct Cars to be on same track
    if(i>0){
      cars[i].m_speed = cars[i-1].m_speed;
      if(!cars[i-1].isOnCurve() && !cars[i].isOnCurve()){
        if(!reverse){
          cars[i].m_position.x = cars[i-1].m_position.x - CAR_LENGTH - CAR_SPACING;
          cars[i].m_position.y = cars[i-1].m_position.y;
        }
        else{
          cars[i].m_position.x = cars[i-1].m_position.x + CAR_LENGTH + CAR_SPACING;
          cars[i].m_position.y = cars[i-1].m_position.y;
        }
        /*
         if(!reverse)
         cars[i].addToTrack(*(cars[i-1].track), cars[i-1].m_position.x - CAR_LENGTH - CAR_SPACING);
         else
         cars[i].addToTrack(*(cars[i-1].track), cars[i-1].m_position.x + CAR_LENGTH + CAR_SPACING);
         
         */
      }
    }
    
    //#SNAP
    //UNCOMMENT THIS EVENTUALLY
    /*
     if(i>0){
     if(!cars[i-1].isOnCurve() && !cars[i].isOnCurve()){
     cars[i].m_position.x = cars[i-1].m_position.x - CAR_LENGTH - CAR_SPACING;
     //if(cars[i-1].track->number != cars[i].track->number){
     //  cars[i].addToNewTrack(cars[i-1].track);
     }
     }
     */
  }
  
}





void Train::render() const{
  Game_Object::render("Locomotive");
  
  for(int i=0; i <numCars; i++){
    cars[i].render();
  }
}

bool AI_Train::collide(const Car & rhs) const{
  if (Car::collide(rhs))
    return true;
  cout<<"Testing AI Cars"<<endl;
  for(int i=0; i< numVisibleCars; i++){
    cout<<"type: "<<cars[i].type<<endl;
    if (cars[i].heuristic_collide(rhs)){
      cout<<"NICK!!!!!"<<endl;
      if(cars[i].collide(rhs)){
        return true;
      }
    }
  }
  cout<<"DONE"<<endl;
  return false;
}

bool AI_Train::heuristic_collide(const Car & rhs) const{
  if(track->number - rhs.track->number < 2){
    cout<<type<<" Car is at "<<m_position.x<<endl;
    cout<<"You are at "<<rhs.m_position.x<<endl;
    if(abs(m_position.x - rhs.m_position.x) < 60.0f){
      cout<<"TRUE TRUEdf TRUE"<<endl;
      return true;
    }
    cout<<"NO"<<endl;
  }
  
  for(int i=0; i< numVisibleCars; i++){
    if (cars[i].heuristic_collide(rhs)){
      return true;
    }
  }
  
  
  return false;
}



AI_Train::AI_Train(Level* level_, const int trackNumber_, const float xCoord_, const float v_0, vector<Car> & cars_, const float timeTrigger_)
: Train(level_, trackNumber_, Point2f(xCoord_, 0.0f), CAR_SIZE, 0.0f)
{
  timeTrigger = timeTrigger_;
  causeForAlarm = true;
  m_speed = v_0;
  if(v_0 < 0){
    reverse = true;
    m_theta = Global::pi;
  }
  int size = (int)cars_.size();
  if(cars_.empty())
    size = 0;
  numVisibleCars = size;
  for(int i=0; i< size; i++){
    cars[i].type = cars_[i].type;
  }
}


