/* This file is part of the Zenipex Library (zenilib).
 * Copyleft (C) 2011 Mitchell Keith Bloch (bazald).
 *
 * This source file is simply under the public domain.
 */

#include <zenilib.h>
#include "Level.h"
#include "Game_Object.h"
#include "Track.h"
#include "Train.h"

#if defined(_DEBUG) && defined(_WINDOWS)
#define DEBUG_NEW new(_NORMAL_BLOCK, __FILE__, __LINE__)
#define new DEBUG_NEW
#endif



using namespace std;
using namespace Zeni;

extern string DebugOutput1;
extern string DebugOutput2;
extern string DebugOutsput3;
extern string DebugOutput4;

bool LAIRD_MODE;

void renderStation(const Point2f position);


/*
class Sound_Emitter {
public:
  Sound_Emitter();
  ~Sound_Emitter();
  
  void step(const float &time_step);
  
private:
  Zeni::Point3f m_position;
  Zeni::Vector3f m_velocity;
  
   Good way of thinking about things.
   * Not the best implementation in a bigger project.
   *
   * You may find that there is a limit to the number of Sound_Sources 
   * you can create in your OpenAL projects.  If this occurs, it would 
   * be a good idea to create a pool of Sound_Sources and to allow 
   * "Emitters" such as this the ability to repurpose a Sound_Source 
   * from the pool when it is time for the Sound to fire.
   *
   * Note further, that there may be NO limit.  Don't try to create 
   * as many Sound_Sources as possible until the limit is hit.  You 
   * may loop until you run out of memory...
   */



/*
  Zeni::Sound_Source *m_sound_source;
};





Sound_Emitter::Sound_Emitter()
: m_position(100.0f, 40.0f, 20.0f), 
m_velocity(-20.0f, -10.0f, -5.0f),
m_sound_source(new Zeni::Sound_Source(get_Sounds()["harmonica"],
                                      1.0f,
                                      1.0f,
                                      m_position,
                                      m_velocity,
                                      true))
{
  m_sound_source->play();
}

Sound_Emitter::~Sound_Emitter() {
  delete m_sound_source;
}

void Sound_Emitter::step(const float &time_step) {
  m_position += time_step * m_velocity;
  m_sound_source->set_position(m_position);
}

*/

class Play_State;
class Instructions_State_Special;
class Level_Selection;


class Title_State_Custom : public Title_State<Level_Selection, Instructions_State_Special> {
public:
  Title_State_Custom();
  
  void render();
};


class Play_State : public Gamestate_Base {
  Play_State(const Play_State &);
  Play_State operator=(const Play_State &);
  
public:
  Play_State(int levelNumber);
  
  
private:
  Level* level;
  Train* playerTrain;
  Color m_prev_clear_color;
  bool m_forward;
  bool m_up;
  bool m_backward;
  bool m_down;
  int m_throttle;
  bool m_brake;
  bool m_translate;
  bool m_crashed;
  bool alerts[14];
  int health;
  float zoomFactor;
  int score;
  int numChickensInThisTrain;
  int numRoostersInThisTrain;
  float last_cleanup;
  Chronometer<Time> m_chrono;
  Chronometer<Time> crashedTimer;
  Chronometer<Time> invincibleTimer;
  Chronometer<Time> alertTimer[14];
  float m_time_passed;
  Point2f upperLeftViewCorner;
  Point2f lowerRightViewCorner;
  Vector2f screenSize;
  
  /*
  Zeni::Time m_current_time;
  float m_time_passed_Audio;
  float m_max_time_step;  //< Optional
  float m_max_time_steps; //< Optional
  Sound_Emitter m_emitter;
   */
  
  
  void on_push();
  void on_pop();
  void prerender();
  void render();
  void on_key(const SDL_KeyboardEvent &event);
  void perform_cleanup();
  void perform_logic();
};








class Instructions_State : public Widget_Gamestate {
  Instructions_State(const Instructions_State &);
  Instructions_State operator=(const Instructions_State &);
  
public:
  Instructions_State()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {
  }
  
private:
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED)
      get_Game().pop_state();
  }
  
  void render() {
    Widget_Gamestate::render();
  }
};



class Instructions_State_Gator : public Widget_Gamestate {
  Instructions_State_Gator(const Instructions_State_Gator &);
  Instructions_State_Gator operator=(const Instructions_State_Gator &);
  
public:
  Instructions_State_Gator()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {
  }
  
private: 
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED){
      get_Game().push_state(new Play_State(3));
    }
  }
  
  void render() {
    Widget_Gamestate::render();
    
    render_image("tutorial5", Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f));
    
  }
};



class Instructions_State_Special4 : public Widget_Gamestate {
  Instructions_State_Special4(const Instructions_State_Special4 &);
  Instructions_State_Special4 operator=(const Instructions_State_Special4 &);
  
public:
  Instructions_State_Special4()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {
  }
  
private: 
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED){
      get_Game().push_state(new Title_State_Custom);
    }
  }
  
  void render() {
    Widget_Gamestate::render();
    
    render_image("tutorial4", Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f));
    
  }
};



class Instructions_State_Special3 : public Widget_Gamestate {
  Instructions_State_Special3(const Instructions_State_Special3 &);
  Instructions_State_Special3 operator=(const Instructions_State_Special3 &);
  
public:
  Instructions_State_Special3()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {
  }
  
private: 
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED){
      get_Game().push_state(new Instructions_State_Special4);
    }
  }
  
  void render() {
    Widget_Gamestate::render();
    
    render_image("tutorial3", Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f));
    
  }
};




class Instructions_State_Special2 : public Widget_Gamestate {
  Instructions_State_Special2(const Instructions_State_Special2 &);
  Instructions_State_Special2 operator=(const Instructions_State_Special2 &);
  
public:
  Instructions_State_Special2()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {
  }
  
private: 
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED){
      get_Game().push_state(new Instructions_State_Special3);
    }
  }
  
  void render() {
    Widget_Gamestate::render();
    
    render_image("tutorial2", Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f));
    
  }
};


class Instructions_State_Special : public Widget_Gamestate {
  Instructions_State_Special(const Instructions_State_Special &);
  Instructions_State_Special operator=(const Instructions_State_Special &);
  
public:
  Instructions_State_Special()
  : Widget_Gamestate(make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)))
  {

  }
  
private:

  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED){
      get_Game().push_state(new Instructions_State_Special2);
    }
  }

  
  void render() {
    Widget_Gamestate::render();
      render_image("tutorial1", Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f));


  }
};









class Level_Selection : public Widget_Gamestate{
  Level_Selection(const Level_Selection &);
  Level_Selection operator=(const Level_Selection &);
  
public:
  Level_Selection()
  : Widget_Gamestate(std::make_pair(Point2f(0.0f, 0.0f), Point2f(800.0f, 600.0f)), true)
  {
    m_widgets.lend_Widget(pop_button1);
    m_widgets.lend_Widget(pop_button2);
    m_widgets.lend_Widget(pop_button3);
    m_widgets.lend_Widget(pop_button4);
    m_widgets.lend_Widget(pop_button5);

  }
  
private:
  class Pop_Button1 : public Text_Button {
    
  public:
    Pop_Button1()
    : Text_Button(Point2f(50.0f, 50.0f), Point2f(250.0f, 100.0f), "system_36_800x600", "Back to Menu")
    { }
    
    void on_accept() {
      get_Game().pop_state();
    }
  }pop_button1;
  
  
  class Pop_Button2 : public Text_Button {
  public:
  Pop_Button2()
  : Text_Button(Point2f(50.0f, 250.0f), Point2f(250.0f, 300.0f), "system_36_800x600", "Level 1")
  { }
  
  void on_accept() {
    get_Game().push_state(new Play_State(1));
  }
}pop_button2;
  
  
  class Pop_Button3 : public Text_Button {
  public:
    Pop_Button3()
    : Text_Button(Point2f(50.0f, 150.0f), Point2f(350.0f, 200.0f), "system_36_800x600", "Toggle Difficulty")
    { }
    
    void on_accept() {
      LAIRD_MODE = !LAIRD_MODE;
      
    }
  }pop_button3;
  
  
  class Pop_Button4 : public Text_Button {
  public:
    Pop_Button4()
    : Text_Button(Point2f(50.0f, 350.0f), Point2f(250.0f, 400.0f), "system_36_800x600", "Level 2")
    { }
    
    void on_accept() {
      get_Game().push_state(new Play_State(2));    
    }
  }pop_button4;
  
  
  
  class Pop_Button5 : public Text_Button {
  public:
    Pop_Button5()
    : Text_Button(Point2f(50.0f, 450.0f), Point2f(250.0f, 500.0f), "system_36_800x600", "Level 3")
    { }
    
    void on_accept() {
      get_Game().push_state(new Instructions_State_Gator());    
    }
  }pop_button5;
  
  
  
  void render() {
    Widget_Gamestate::render();
    
    if(LAIRD_MODE){
      Zeni::Font &fr = get_Fonts()["small"];
      fr.render_text("Laird Mode is On",
                     Point2f(400.0f, 175.0f - 0.5f * fr.get_text_height()),
                     get_Colors()["title_text"]);
    }
    else{
      Zeni::Font &fr = get_Fonts()["small"];
      fr.render_text("Laird Mode is Off",
                     Point2f(400.0f, 175.0f - 0.5f * fr.get_text_height()),
                     get_Colors()["title_text"]);
    }
  }
};








class Game_Over_Screen : public Instructions_State{
  Game_Over_Screen(const Game_Over_Screen &);
  Game_Over_Screen operator=(const Game_Over_Screen &);
  
public:
  Game_Over_Screen()
  : Instructions_State()
  {
  }
  
private:
  Chronometer<Time> time;

  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED)
      get_Game().pop_state();
  }
  
  void perform_logic(){
	  if(! time.is_running())
      time.start();
	  if(time.seconds() > 3.0f)
      get_Game().push_state(new Title_State_Custom);
    
  }
  
  void render() {
    Widget_Gamestate::render();
    Zeni::Font &fr = get_Fonts()["title"];
    
    fr.render_text("Game Over",
                   Point2f(400.0f, 300.0f - 0.5f * fr.get_text_height()),
                   get_Colors()["title_text"],
                   ZENI_CENTER);
  }
};





class Level_Complete_Screen : public Instructions_State{
  Level_Complete_Screen(const Level_Complete_Screen &);
  Level_Complete_Screen operator=(const Level_Complete_Screen &);
  
public:
  Level_Complete_Screen(int score_)
  : Instructions_State()
  {
    score = score_;
  }
  
private:
  int score;
  Chronometer<Time> time;
  
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED)
      get_Game().pop_state();
  }
  
  void perform_logic(){
	  if(! time.is_running())
      time.start();
	  if(time.seconds() > 3.0f)
      get_Game().push_state(new Title_State_Custom);
    
  }
  
  void render() {
    Widget_Gamestate::render();
    Zeni::Font &fr = get_Fonts()["title"];
    
    fr.render_text("Level Complete!",
                   Point2f(400.0f, 300.0f - 0.5f * fr.get_text_height()),
                   get_Colors()["title_text"],
                   ZENI_CENTER);
    fr.render_text("Score: " + itoa(score),
                   Point2f(400.0f, 400.0f - 0.5f * fr.get_text_height()),
                   get_Colors()["title_text"],
                   ZENI_CENTER);
  }
};






class Cut_Scene_One : public Instructions_State{
  Cut_Scene_One(const Cut_Scene_One &);
  Cut_Scene_One operator=(const Cut_Scene_One &);
  
public:
  Cut_Scene_One()
  : Instructions_State()
  {
  }
  
private:
	Chronometer<Time> time;
  void on_key(const SDL_KeyboardEvent &event) {
    if(event.keysym.sym == SDLK_ESCAPE && event.state == SDL_PRESSED)
      get_Game().pop_state();
  }
  
  void perform_logic(){
	  if(! time.is_running())
      time.start();
	  if(time.seconds() > 5.0f)
      get_Game().push_state(new Play_State(1));
    
  }
  
  void render() {
    Widget_Gamestate::render();
    Zeni::Font &fr = get_Fonts()["title"];
    
    fr.render_text("Level 1",
                   Point2f(400.0f, 300.0f - 0.5f * fr.get_text_height()),
                   get_Colors()["title_text"],
                   ZENI_CENTER);
  }
};
















Play_State::Play_State(int levelNumber):
m_prev_clear_color(get_Video().get_clear_Color()),
m_forward(false),
score(0),
m_up(false),
m_backward(false),
m_down(false),
m_time_passed(0.0f),
m_throttle(0),
m_brake(false),
m_translate(false),
m_crashed(false),
zoomFactor(1.5),
last_cleanup(0.0f),
numChickensInThisTrain(0),
numRoostersInThisTrain(0)

/*
m_time_passed_Audio(0.0f),
m_max_time_step(1.0f / 20.0f), // make the largest physics step 1/20 of a second
m_max_time_steps(10.0f) // allow no more than 10 physics steps per frame
 */

{
  
  
  
  //m_widgets.lend_Widget(m_bgm_volume_slider);
  
  /*
  Sound &sr = get_Sound();
  
  sr.set_BGM("harmonica");
  sr.set_BGM_looping(true);
  sr.play_BGM();
  */
  
  
  if (LAIRD_MODE){
    health = 25;
  }
  else
    health = 4;
  
  
  for(int i=0; i<14; i++){
    alerts[i] = false;
  }
  screenSize = Vector2f(get_Window().get_width(), get_Window().get_height());
  upperLeftViewCorner = Point2f(0.0f,0.0f);
  lowerRightViewCorner = Point2f(get_Window().get_width()-100.0f, get_Window().get_height());
  set_pausable(true);
  m_chrono.start();
  level = new Level(levelNumber);
  playerTrain = level->playerTrain;
}




void Play_State::on_push() {
  //get_Window().mouse_grab(true);
  get_Window().mouse_hide(true);
  get_Video().set_clear_Color(Color(0.0f, 0.0f, 0.0f, 0.0f));
} 

void Play_State::on_pop() {
  //get_Window().mouse_grab(false);
  get_Window().mouse_hide(false);
  get_Video().set_clear_Color(m_prev_clear_color);
}



void Play_State::prerender(){
  screenSize = Vector2f(get_Window().get_width(), get_Window().get_height());
  Point2f positionOfPlayer = playerTrain->get_position();
  upperLeftViewCorner.x = positionOfPlayer.x - (screenSize.x/2);
  lowerRightViewCorner.x = positionOfPlayer.x + zoomFactor*screenSize.x/2;//positionOfPlayer.x + width/2 -50.0f;
  
}

void Play_State::render() {
  get_Video().set_2d(make_pair(upperLeftViewCorner , lowerRightViewCorner), true);
  level->render();
  playerTrain->render();
  for(list<Parked_Car *>::const_iterator it = level->parkedCars.begin(); it != level->parkedCars.end(); it++){
    (*it)->render();
  }
  for(list<AI_Train *>::const_iterator it = level->AITrains.begin(); it != level->AITrains.end(); it++){
    (*it)->render();
  }
  
  int numTracks = level->numTracks;
  for(int i=0; i< numTracks; i++){
    int numStations = (int) level->tracks[i].stationLocations.size();
    for(int j = 0; j < numStations; j++){
      renderStation(level->tracks[i].stationLocations[j]);
    }
  }
  
  //float height = screenSize.y;
  float width = screenSize.x;
  Vector2f displayedSize = lowerRightViewCorner - upperLeftViewCorner;
  
  
  
  get_Video().set_2d(make_pair(Point2f(0.0f, 0.0f), displayedSize),true); //Point2f(screenSize.x, 400.0f)), true);
  Fonts &fr = get_Fonts();
  Font &font = fr["system_36_500x600"]; //or a fonts.xml entry of your choice
  font.render_text("FPS: " + ulltoa(get_Game().get_fps()), Vector2f(0.7*width, 0.0f), Color());
  font.render_text(itoa(m_chrono.seconds()/60) + ":"+ itoa((int)m_chrono.seconds()%60), Vector2f(0.9*width, 0.0f), Color());
  font.render_text("Throttle: " + itoa(m_throttle), Vector2f(400.0f, level->numTracks*TRACK_SPACING+30.0f), Color());
  font.render_text("Health: " + itoa(health), Vector2f(400.0f, level->numTracks*TRACK_SPACING+2*30.0f), Color());
  font.render_text("Score: " + itoa(score), Vector2f(400.0f, level->numTracks*TRACK_SPACING+3*30.0f), Color());
  /*
   font.render_text(DebugOutput1.c_str(), Vector2f(0.0f, 400.0f), Color());
   font.render_text(DebugOutput2.c_str(), Vector2f(0.0f, 430.0f), Color());
   font.render_text(DebugOutput3.c_str(), Vector2f(0.0f, 460.0f), Color());
   font.render_text(DebugOutput4.c_str(), Vector2f(0.0f, 490.0f), Color());
   */
  Point2f lightPoint = Point2f(displayedSize.x-75.0f, 40.0f);
  Point2f lightSize = Vector2f(75.0f, 45.0f);
  Point2f spacing = Vector2f(0.0f, TRACK_SPACING);
  
  for(int i=0; i<level->numTracks; i++){
    if(alerts[i])
      render_image("Light_Amber", lightPoint+i*spacing, lightPoint+lightSize+i*spacing);
    else
      render_image("Light_Green", lightPoint+i*spacing, lightPoint+lightSize+i*spacing);
  }
  
  Point2f backLightPoint = Point2f(0.0f, 40.0f);
  for(int i=0; i<level->numTracks; i++){
    if(alerts[i+5])
      render_image("Light_Amber", backLightPoint+i*spacing, backLightPoint+lightSize+i*spacing);
    else
      render_image("Light_Green", backLightPoint+i*spacing, backLightPoint+lightSize+i*spacing);
  }
  
  
  
}






void Play_State::on_key(const SDL_KeyboardEvent &event) {
  switch(event.keysym.sym) {
    case SDLK_w:
      m_forward = event.type == SDL_KEYDOWN;
      break;
      
    case SDLK_UP:
      m_up = event.type == SDL_KEYDOWN;
      break;
      
    case SDLK_DOWN:
      m_down = event.type == SDL_KEYDOWN;
      //m_translate = event.type == SDL_KEYDOWN;
      break;
      
    case SDLK_SPACE:
      m_brake = event.type == SDL_KEYDOWN;
      break;
      
    case SDLK_RETURN:
      m_forward = event.type == SDL_KEYDOWN;
      break;
      
    case SDLK_0:
      m_throttle = 0;
      break;
    case SDLK_a:
      m_throttle = 1;
      break;
    case SDLK_s:
      m_throttle = 2;
      break;
    case SDLK_d:
      m_throttle = 3;
      break;
    default:
      Gamestate_Base::on_key(event); //Let Gamestate_Base handle it
      break;
  }
}



void Play_State::perform_cleanup(){
  level->filterAITrains();
  level->filterParkedCars();
}

void Play_State::perform_logic() {
  const float time_passed = m_chrono.seconds();
  const float time_step = time_passed - m_time_passed;
  m_time_passed = time_passed;
  
  
  
  /*
  {// Update time_passed
    const Time temp_time = get_Timer().get_time();
    m_time_passed_Audio += temp_time.get_seconds_since(m_current_time);
    m_current_time = temp_time;
  }
  
   Simple update
   *
   * Doing more than this for Sound is pretty pointless.  In fact, 
   * I would special case Sound updates in a real game to be updated 
   * in this manner even if physics updates are done on a bounded 
   * step size.
   
  {
    m_emitter.step(m_time_passed);
    m_time_passed = 0.0f;
  }
  
  */
  
  
  //DO LOGIC ON PLAYER INPUT
  if(!m_crashed){
    TrainInfoPack trainData;
    
    switch(m_throttle){
      case 0:
        trainData.a = -10.0f;
        trainData.v_term = 500.0f;
        break;
      case 1:
        trainData.a = 50.0f;
        trainData.v_term = 100.0f;
        break;
      case 2:
        trainData.a = 70.0f;
        trainData.v_term = 200.0f;
        break;
      case 3:
        trainData.a = 80.0f;
        trainData.v_term = 300.0f;
      default:
        break;
    }
    
    if (m_brake){
      trainData.a = -0.1f*playerTrain->m_speed -50.0f;
      m_throttle = 0;
    }
    
    trainData.t = time_step;
    if(m_up)
      trainData.command = GO_UP;
    else if (m_down)
      trainData.command = GO_DOWN;
    else
      trainData.command = GO_STRAIGHT;
    
    trainData.completedSwitch = false;
    trainData.isPlayerTrain = true;
    playerTrain->moveAlongTrack(trainData);
  }
  
  
  
  //MOVE PARKED CARS
  for(list<Parked_Car *>::iterator it = level->parkedCars.begin(); it != level->parkedCars.end(); ++it){
    TrainInfoPack parkedData;
    parkedData.t = time_step;
    
    (*it)->moveCarAlong(parkedData);
  }
  
  
  
  //MOVE AI TRAINS
  for(list<AI_Train *>::iterator it = level->AITrains.begin(); it != level->AITrains.end(); ++it){
    TrainInfoPack AIData;
    AIData.t = time_step;
    
    (*it)->moveAlongTrack(AIData);
  }
  
  
  
  //HANDLE CAR SCORING
  //TODO: Is there code that relies on numVisibleCars that I should worry about?
  
  
  for(int i=0; i< playerTrain->numCars; i++){
    bool scored = playerTrain->cars[i].track->checkForStationScore(playerTrain->cars[i].m_position.x);
    
    
    
    if(level->levelNumber != 3){
      if (playerTrain->cars[i].type != INVIS){
        if (scored){
          Car_Type carType = playerTrain->cars[i].type;
          switch(carType){
            case CHICKEN:
              score += 10;
              numChickensInThisTrain++;
              break;
            case SHEEP:
              score += 20;
              break;
            case CATTLE:
              score += 50;
              break;
            case ROOSTER:
              numRoostersInThisTrain++;
              break;
            default:
              break;
          }
          playerTrain->removeCar(i);
        }
      }
      if(i == playerTrain->numCars-1 && scored){
        score+= (numChickensInThisTrain)*(numRoostersInThisTrain * 2)*10;
        numChickensInThisTrain = 0;
        numRoostersInThisTrain = 0;
        //RESET THESE VARS
      }
    }
    else{
      if(scored){
        Car_Type carType = playerTrain->cars[i].type;
        switch(carType){
          case INVIS:
            break;
          case GATOR:
            score +=50;
            playerTrain->removeCar(i);
            break;
          default:
            score-= 200;
            playerTrain->removeCar(i);
            break;
        }
      }
    }
  }
  
  
  
  //CLEANUP PERIODICALLY
  if (abs(time_passed-last_cleanup) > 5){
    last_cleanup = time_passed;
    perform_cleanup();
  }
  
  
  //HANDLE COLLISIONS
  playerTrain->collide(level->parkedCars);
  
  
  bool collided = playerTrain->collide(level->AITrains);
  if (collided){
    health--;
    if(health <= 0){
      crashedTimer.start();
      playerTrain->m_speed = 0;
      m_crashed = true;
    }
  }
  if(crashedTimer.seconds() > 3.0f){
    get_Game().push_state(new Game_Over_Screen());
    //get_Game().pop_state();
  }
	
  
  //level->triggerForTime(m_chrono.seconds(), alerts);
  level->triggerForX(playerTrain->m_position.x);
  level->triggerForTime(playerTrain->m_position.x,alerts);
  
  
  
  //RESET TIMERS and START THEM FOR ALERTS
  for(int i = 0; i< 14; i++){
    if(alertTimer[i].seconds() > 5.0f){
      alertTimer[i].stop();
      alertTimer[i].reset();
      alerts[i] = false;
    }
    if(alerts[i] && ! alertTimer[i].is_running()){
      alertTimer[i].start();
    }
  }
  
  
  if(abs(playerTrain->m_position.x - level->levelEnd) <  8*TRACK_WIDTH){
    get_Game().push_state(new Level_Complete_Screen(score));
  }
  
  
}























Title_State_Custom::Title_State_Custom()
  : Title_State<Level_Selection, Instructions_State_Special>(" ")
  {
    m_widgets.unlend_Widget(title);
  }
  
void Title_State_Custom::render() {
  Title_State<Level_Selection, Instructions_State_Special>::render();
  //get_Video().set_clear_Color(Color(1.0f, 0.26f, 0.13f, 0.0f));
  render_image("logo", Point2f(150.0f, 25.0f), Point2f(650.0f, 225.0f));
}








class Bootstrap {
  class Gamestate_One_Initializer : public Gamestate_Zero_Initializer {
    virtual Gamestate_Base * operator()() {
      Window::set_title("zenilib Application");
      
      get_Joysticks();
      get_Video();
      get_Textures();
      get_Fonts();
      get_Sounds();
      
      return new Title_State_Custom;//Title_State<Play_State, Instructions_State>("Zenipex Library\nApplication");
    }
  } m_goi;
  
public:
  Bootstrap() {
    g_gzi = &m_goi;
  }
} g_bootstrap;





int main(int argc, char **argv) {
  LAIRD_MODE = false;
  return zenilib_main(argc, argv);
}





void renderStation(const Point2f position){
  render_image(
               "Station",
               position,
               position + Vector2f(TRACK_WIDTH+25, TRACK_HEIGHT+25),
               0,
               1.0f,
               position,
               false,
               Color());
}
