//
//  Track.h
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#ifndef TRACK
#define TRACK

#include "Game_Object.h"
#include "Train.h"

const static float TRACK_HEIGHT = 25.0f;
const static float TRACK_WIDTH = 128.0f;
const static float TRACK_SPACING = 50.0f;

class TrackPiece;
class Track;

enum TrackTypeEnum {STRAIGHT, SWITCH_UP, SWITCH_DOWN, STATION, SWITCH_TRIPLE, END_UP, END_DOWN, INVISIBLE, ENDOFLINE};


class Track{
private:
  int numTrackPieces;
  Point2f upperLeftCorner;
  Point2f lowerRightCorner;
  
public:
  vector<Point2f> stationLocations;
  TrackPiece* trackPieces;
  void addTrackPiece(int index, TrackTypeEnum type_);
  float yCoord;
  int number;
  

  
public:
  Track();
  Track(const Track & track);
  Track& operator=(const Track & track);
  Track(int assignedNumber, float horizontalCoord, int numTrackPieces_);
  void setViewWindow(Point2f upperLeftPoint, Point2f lowerRightPoint);
 void moveAlong(TrainInfoPack & trainData);
  int checkForStationScore(float xCoord);

  virtual void render() const;
  
  int calculateTrackPieceNum(float X);
  float calculateLocalX(float X);
  void copyAll(const Track & track);
  ~Track();
};




class TrackPiece: public Game_Object{
public:
  TrackTypeEnum type;
  TrackPiece();
  TrackPiece(const Point2f &position_, TrackTypeEnum type_);
  void render() const;
  void moveAlong(TrainInfoPack & trainData, float localX);
  ~TrackPiece();
  
private:
  bool setSwitch;
  float switchedAtX;
  bool canSwitch(float localX);
  float getTheta(float localX);
};


/*
class SwitchPiece: public TrackPiece {
private:
  bool switchDown;
  bool switchUp;
  
public:
  SwitchPiece(const Point2f &position_, const Vector2f &size_ = Vector2f(WIDTH, 2*HEIGHT+SPACING),
              const float &theta_ = 0.0f, bool allowUpSwitch, bool allowDownSwitch);
  
  virtual float angleForXCoordAndCommand(float XCoord, Train_Command command);
};


*/


#endif
