//
//  Train.h
//  Application
//
//  Created by Stephen Brown on 9/20/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#ifndef TRAIN
#define TRAIN

#include <iostream>
#include "Game_Object.h"

using namespace Zeni;

const float CAR_LENGTH = 50.0f;
const float CAR_HEIGHT = 25.0f;
const float CAR_SPACING = 5.0f;
const float MAX_CARS = 12;
const Vector2f CAR_SIZE(50.0f, 25.0f);

class Track;
class Level;
class AI_Train;

enum Train_Command {GO_STRAIGHT, GO_UP, GO_DOWN};
enum Car_Type {INVIS, CHICKEN, SHEEP, CATTLE, ROOSTER, GATOR, NONE};

struct TrainInfoPack{
  Train_Command command;
  Point2f position;
  float v;
  float a;
  float t;
  float theta;
  float v_term;
  bool completedSwitch;
  bool isLocomotive;
  bool isPlayerTrain;
  
  TrainInfoPack() {
    position = Point2f(0,0);
    a = 0;
    t = 0;
    theta = 0;
    v_term = 20.0f;
    command = GO_STRAIGHT;
    completedSwitch = false;
    isLocomotive = false;
    isPlayerTrain = false;
  }
  
};





class Car : public Game_Object{ 
public:
  float offset;
  float timeTrigger;
  bool reverse;
  Level* level;
  Track* track;
  Car_Type type;
  bool causeForAlarm;
  Car();
  Car(Level* level_, const Point2f &position_, const float &theta_, const float timeTrigger_);
  Car(Car_Type type_);
  int addToTrack(Track & track, float verticalCoord);
  void addToNearestTrack(Level* level);
  void addToNewTrack(Track * track_);
  void moveToPosition(float X, float Y, float theta);
  bool isOnCurve();
  void moveCarAlong(TrainInfoPack & trainData);
  virtual bool collide(const Car & rhs) const;
  virtual bool heuristic_collide(const Car & rhs) const;
  virtual const float get_radius() const;
  float get_radiusForAngle(float alpha) const;
  virtual void render() const;
};



class Parked_Car : public Car{
public:
  Parked_Car(Level* level_, const int trackNumber_, const float xCoord_, const float v_0, Car_Type type_, const float timeTrigger_);
  //Parked_Car(const Point2f &position_, const Vector2f &size_, const float &theta_);
  
  //void render() const;
};





class Train : public Car{
public:
  bool crashed;
  int numVisibleCars;
  vector<Car> cars;
  int numCars;
  
  virtual void render() const;
  Train();
  Train(Level* level_, const int trackNumber_, const Point2f &position_, const Vector2f &size_, const float &theta_);
  void collide(list<Parked_Car *> & parkedCars);
  bool collide(list<AI_Train *> & AITrains);
  void moveCars(TrainInfoPack & trainData);
  void moveAlongTrack(TrainInfoPack & trainData);
  void spawnCar(Car_Type type_ = INVIS);
  void addRealCar(Car_Type type_);
  void addCar(Car_Type type_);
  Car_Type removeCar(int index);
};

class AI_Train: public Train{
public:
  virtual bool collide(const Car & rhs) const;
  virtual bool heuristic_collide(const Car & rhs) const;
  AI_Train(Level* level_, const int trackNumber_, const float xCoord_, const float v_0, vector<Car> & cars_, const float timeTrigger_);
  
};

class Train_Player : public Train{
  
  
};


#endif